searchData={"content_type":"text/markdown","items":[{"doc":"Bio.Random.Dna will generate random sequences of DNA. Sequences are written to\na file with 1 per line, and are separated by a `\n` character.","ref":"Mix.Tasks.Bio.Random.Dna.html","title":"mix bio.random.dna","type":"task"},{"doc":"* `--seed/-s` - RNG seed (defaults to RNG default seeding)\n\n* `--algorithm/-a` - RNG algorithm (defaults to exsss)\n\n* `--seq-count/-c` - integer number of sequences to generate (required)\n\n* `--seq-size/-z` - integer size of sequence to generate (required)\n\n* `--outfile/-f` - output filename or path (default: random_sequences.txt)","ref":"Mix.Tasks.Bio.Random.Dna.html#module-command-line-options","title":"Command line options - mix bio.random.dna","type":"task"},{"doc":"$ mix bio.random.dna -s 0 -c 100 -z 50\n\nThis would write 100 sequences of 50 nucleotides to a file called\n`random_sequences.txt`.\n\n    $ mix bio.random.dna -s 0 -c 100 -z 50 -f my_random_sequences.txt\n\nThis would write the same 100 sequences of 50 nucleotides to a file called\n`my_random_sequences.txt`.","ref":"Mix.Tasks.Bio.Random.Dna.html#module-examples","title":"Examples - mix bio.random.dna","type":"task"},{"doc":"","ref":"Mix.Tasks.Bio.Random.Dna.html#run/1","title":"Mix.Tasks.Bio.Random.Dna.run/1","type":"function"},{"doc":"Implementations of the basic sequence functionality.\n\nCalling `use Bio.BaseSequence` will generate a simple struct in the calling\nmodule, as well as the implementation for the `Enumerable` protocol.\n\nBecause the `Enum` module makes certain assumptions about the data that it is\ngiven, we cannot trust that the functions therein will always behave how it\nmakes the most sense. As an example, there is no way to ensure that\n`Enum.slide/3` returns anything other than a list. I believe that it makes\nsense for it to return the enumerable type, so you would get e.g. a\n`Bio.Sequence.DnaStrand` back.\n\nWith that said, many of the `Enum` module's functions _shouldn't_ make\nassumptions. This is largely idiosynctratic, and so instead of trying to\nham-fist the `Enum` functions to work, I just wrapped them up with `Bio.Enum`.\n\nThe implementations in `Bio.Enum` rely on the `Enum` functions to work, but\nthey go the extra mile in terms of returning things that seem to make the most\nsense. See the documentation of `Bio.Enum` for more on that.\n\nThis module will also cause `new/2` to be defined. This function takes a\nsequence as well as the keywords `:label` and `:length`. For more examples of\nusing `new/2` see `Bio.Sequence.AminoAcid`, `Bio.Sequence.DnaStrand`, or\n`Bio.Sequence.RnaStrand`.","ref":"Bio.BaseSequence.html","title":"Bio.BaseSequence","type":"module"},{"doc":"Deals with conversions between polymers.\n\nThe sequences that this will work with must define an implementation for the\n`Bio.Polymeric` protocol. This is then used with the definition of\nthe `to/1` callbacks for the `Bio.Convertible` behaviour. These will\nbe given the kmer enumeration that they define with that function.\n\nThis module wraps the logic of accessing a given polymer's defined\nconversions. The primary idea is that I wanted to expose the ability to\nprovide a non-default conversion without losing the semantics of a simple\ndefault when it's present.\n\nTo put that in more concrete terms, I wanted this to be viable:\n\n    iex>dna = DnaStrand.new(\"ttagccgt\", label: \"a label\")\n    ...>Bio.Polymer.convert(dna, RnaStrand)\n    {:ok, %RnaStrand{sequence: \"uuagccgu\", length: 8, label: \"a label\"}}\n\nBut, and this is the important part, other conversions are not well defined by\ndefaults. For example:\n\n    iex>amino = AminoAcid.new(\"maktg\")\n    ...>Bio.Polymer.convert(amino, DnaStrand)\n    {:error, :undef_conversion}\n\nThe `:undef_conversion` indicates that there is no viable default\nimplementation of the conversion between these polymers. It _does not_\nindicate that there is none. Obviously one can convert from an amino acid to\n_some_ DNA strand. However, because this would imply making a selection from\nthe available codons, that is left to the logic of whatever application is\ndoing so.\n\nThe way that you would do that is straight forward, you would define a\nconversion module and pass it to the `convert/3` function as the keyword\nargument `:conversion`. For example, if we wanted to defined a mapping that\nconverted into a compressed DNA representation, we could do:\n\n    iex>defmodule CompressedAminoConversion do\n    ...>  use Bio.Convertible do\n    ...>    def to(DnaStrand), do: {:ok, &compressed/2, 1}\n    ...>  end\n    ...>\n    ...>  def compressed({:ok, knumerable, data}, _) do\n    ...>    data = data\n    ...>           |> Map.drop([ :length ])\n    ...>           |> Map.to_list()\n    ...>    knumerable\n    ...>    |> Enum.map(&to_codon/1)\n    ...>    |> Enum.join(\"\")\n    ...>    |> DnaStrand.new(data)\n    ...>  end\n    ...>\n    ...>  defp to_codon(aa) do\n    ...>    case aa do\n    ...>      \"a\" -> \"gcn\"\n    ...>      \"r\" -> \"cgn\"\n    ...>      \"n\" -> \"aay\"\n    ...>      \"d\" -> \"gay\"\n    ...>      \"c\" -> \"tgy\"\n    ...>      \"e\" -> \"gar\"\n    ...>      \"q\" -> \"car\"\n    ...>      \"g\" -> \"ggn\"\n    ...>      \"h\" -> \"cay\"\n    ...>      \"i\" -> \"ath\"\n    ...>      \"l\" -> \"ctn\"\n    ...>      \"k\" -> \"aar\"\n    ...>      \"m\" -> \"atg\"\n    ...>      \"f\" -> \"tty\"\n    ...>      \"p\" -> \"ccn\"\n    ...>      \"s\" -> \"tcn\"\n    ...>      \"t\" -> \"acn\"\n    ...>      \"w\" -> \"tgg\"\n    ...>      \"y\" -> \"tay\"\n    ...>      \"v\" -> \"gtn\"\n    ...>    end\n    ...>  end\n    ...>end\n    ...>amino = AminoAcid.new(\"maktg\", label: \"polypeptide-∂\")\n    ...>Bio.Polymer.convert(amino, DnaStrand, conversion: CompressedAminoConversion)\n    {:ok, %DnaStrand{sequence: \"atggcnaaracnggn\", length: 15, label: \"polypeptide-∂\"}}\n\nThis is made possible because of the simple implementation of the\n`Bio.Polymeric` interface for the `Bio.Sequence.AminoAcid`. If\nyou want to define your own convertible polymer types, you can. It requires\ndefining the module and the implementation of `convert/1`. You can read the\n`Bio.Sequence.AminoAcid` source for more clarity on the details.\n\nThis package attempts to define reasonable defaults for all the occasions\nwhich it can. This includes converting DNA into RNA, and RNA to DNA. The\nconversions from DNA/RNA to Amino Acid are done using standard codon tables.\n\nThe Conversion module idea is provided as an escape hatch for more particular\napplications which may require bespoke logic. An example would be converting\nAmino Acids into a DNA sequence, as above. There are likely more use cases\nthan I could possibly compile on my own, so I tried to come up with a way to\nalleviate that pressure.","ref":"Bio.Polymer.html","title":"Bio.Polymer","type":"module"},{"doc":"Apply a conversion to a given datum.\n\nThe `convert/3` function is at the core of using the `Bio.Polymer`\nmodule. By passing the function a struct and the module you wish to convert\nto, you are hooking into the underlying implementation of the\n`Bio.Convertible` for that module. This means that both the struct\nyou given _as well as the module_ must have this implemented.\n\n# Examples\nGiven a struct and module with a known conversion:\n\n    iex>dna = DnaStrand.new(\"ttagccgt\", label: \"a label\")\n    ...>Bio.Polymer.convert(dna, RnaStrand)\n    {:ok, %RnaStrand{sequence: \"uuagccgu\", length: 8, label: \"a label\"}}\n\nGiven a struct and module with unknown conversions:\n\n    iex>amino = AminoAcid.new(\"maktg\")\n    ...>Bio.Polymer.convert(amino, DnaStrand)\n    {:error, :undef_conversion}\n\nGiven a struct that doesn't implement `Bio.Sequential`:\n\n    iex>Bio.Polymer.convert(%SomeModule{}, DnaStrand)\n    {:error, :no_converter}","ref":"Bio.Polymer.html#convert/3","title":"Bio.Polymer.convert/3","type":"function"},{"doc":"","ref":"Bio.Polymer.html#valid?/2","title":"Bio.Polymer.valid?/2","type":"function"},{"doc":"Validate a given sequence struct according to its `Bio.Polymeric` implementation.","ref":"Bio.Polymer.html#validate/2","title":"Bio.Polymer.validate/2","type":"function"},{"doc":"`Bio.Sequence` is the basic building block of the sequence types.\n\nThe core concept here is that a polymer is a sequence of elements encoded as a\nbinary. This is stored in the base `%Bio.Sequence{}` struct, which has both a\n`sequence` and `length` field, and may carry a `label` and `alphabet` field as\nwell.\n\nThe struct is intentionally sparse on information since this is meant to\ncompose into larger data types. For example, the `Bio.Sequence.DnaDoubleStrand` struct,\nwhich has two polymer `Bio.Sequence.DnaStrand`s as the `top_strand` and\n`bottom_strand` fields.\n\nBecause many of the sequence behaviors are shared, they are implemented by\n`Bio.BaseSequence` and used in the modules that need them. This allows us to\nensure that there is a consistent implementation of the `Enumerable` protocol,\nwhich in turn allows for common interaction patterns _a la_ Python strings:\n\n    iex>\"gmc\" in Bio.Sequence.new(\"agmctbo\")\n    true\n\n    iex>Bio.Sequence.new(\"agmctbo\")\n    ...>|> Enum.map(&(&1))\n    [\"a\", \"g\", \"m\", \"c\", \"t\", \"b\", \"o\"]\n\nMy hope is that this alleviates some of the pain of coming from a language\nwhere strings are slightly more complex objects.\n\nAdditionally, you should look at the `Bio.Enum` module for dealing with cases\nwhere the `Enum` default implementation results in odd behavior. It also\nimplements certain behaviors like returning the same type for functions:\n\n    iex>Bio.Sequence.new(\"agmctbo\")\n    ...>|> Enum.slice(2, 2)\n    'mc'\n\n\nvs\n\n    iex>alias Bio.Enum, as: Bnum\n    ...>Bio.Sequence.new(\"agmctbo\")\n    ...>|> Bnum.slice(2, 2)\n    %Bio.Sequence{sequence: \"mc\", length: 2}","ref":"Bio.Sequence.html","title":"Bio.Sequence","type":"module"},{"doc":"Get the full name for a given monomer.\n\n# Example\n    iex>MonomerName.nucleic_acid(\"a\")\n    \"adenine\"\n\n    iex>MonomerName.amino_acid(\"a\")\n    \"alanine\"","ref":"Bio.Sequence.MonomerName.html","title":"Bio.Sequence.MonomerName","type":"module"},{"doc":"Mapping amino acids to their chemical names","ref":"Bio.Sequence.MonomerName.html#amino_acid/1","title":"Bio.Sequence.MonomerName.amino_acid/1","type":"function"},{"doc":"iex>MonomerName.nucleic_acid(\"a\")\n    \"adenine\"","ref":"Bio.Sequence.MonomerName.html#amino_acid/1-example","title":"Example - Bio.Sequence.MonomerName.amino_acid/1","type":"function"},{"doc":"Mapping nucleotides to their chemical names","ref":"Bio.Sequence.MonomerName.html#nucleic_acid/1","title":"Bio.Sequence.MonomerName.nucleic_acid/1","type":"function"},{"doc":"iex>MonomerName.nucleic_acid(\"a\")\n    \"adenine\"","ref":"Bio.Sequence.MonomerName.html#nucleic_acid/1-example","title":"Example - Bio.Sequence.MonomerName.nucleic_acid/1","type":"function"},{"doc":"A module for working with DNA.\n\nThis module doesn't contain a representative struct, as with `Bio.Sequence.Rna`.\nThis is because there are multiple ways to interpret a string as DNA. Namely, it\ncan either be single or double stranded. This is why the\n`Bio.Sequence.DnaStrand` and `Bio.Sequence.DnaDoubleStrand` modules exist.\n\nHowever, this is the interface for dealing with things like `complement/1` and\n`reverse_complement!/1`.\n\nAdditionally, this module handles defining default conversions for the DNA\nsequence types into RNA sequence types (`Bio.Sequence.RnaStrand` and\n`Bio.Sequence.RnaDoubleStrand`). Conversions defined here are used by the\n`Bio.Sequence.DnaStrand` and `Bio.Sequence.DnaDoubleStrand` modules.\n\nThe default conversions use conventional nucleotides and map them to their\nrelevant RNA nucleotides:\n\n```\na -> a\nt -> u\ng -> g\nc -> c\n```\n\nCasing is preserved, so mixed case sequences will not be altered.\n\n# Example\n\n    iex>DnaStrand.new(\"taTTg\")\n    ...>|> Bio.Polymer.convert(RnaStrand)\n    {:ok, %RnaStrand{sequence: \"uaUUg\", length: 5}}\n\nThis is guaranteed, so you may encode these with intention and assume that\nthey are preserved across conversions.","ref":"Bio.Sequence.Dna.html","title":"Bio.Sequence.Dna","type":"module"},{"doc":"Provide the DNA complement to a sequence.\n\nGiven a sequence that is either a binary or a `Bio.Sequence.DnaStrand`,\nreturns the DNA complement as defined by the standard nucleotide complements.\n\n# Examples\n    iex>Dna.complement(\"attgacgt\")\n    {:ok, \"taactgca\"}\n\n    iex>DnaStrand.new(\"attgacgt\")\n    ...>|> Dna.complement()\n    {:ok, %DnaStrand{sequence: \"taactgca\", length: 8, alphabet: Bio.Sequence.Alphabets.Dna.common()}}","ref":"Bio.Sequence.Dna.html#complement/2","title":"Bio.Sequence.Dna.complement/2","type":"function"},{"doc":"Provide the DNA reverse complement to a sequence.\n\nGiven a sequence that is either a binary or a `Bio.Sequence.DnaStrand`,\nreturns the DNA reverse complement as defined by the standard nucleotide\ncomplements.\n\n# Examples\n    iex>Dna.reverse_complement!(\"attgacgt\")\n    \"acgtcaat\"\n\n    iex>DnaStrand.new(\"attgacgt\")\n    ...>|> Dna.reverse_complement!()\n    %DnaStrand{sequence: \"acgtcaat\", length: 8}","ref":"Bio.Sequence.Dna.html#reverse_complement!/2","title":"Bio.Sequence.Dna.reverse_complement!/2","type":"function"},{"doc":"","ref":"Bio.Sequence.Dna.html#t:complementable/0","title":"Bio.Sequence.Dna.complementable/0","type":"type"},{"doc":"","ref":"Bio.Sequence.Dna.html#t:msg/0","title":"Bio.Sequence.Dna.msg/0","type":"type"},{"doc":"A representative struct for Double Stranded DNA polymers.","ref":"Bio.Sequence.DnaDoubleStrand.html","title":"Bio.Sequence.DnaDoubleStrand","type":"module"},{"doc":"Generate a new `%Bio.Sequence.DnaDoubleStrand{}` struct.","ref":"Bio.Sequence.DnaDoubleStrand.html#new/2","title":"Bio.Sequence.DnaDoubleStrand.new/2","type":"function"},{"doc":"`label` - This is a label applied to the top and bottom.\n`alphabet` - This is the alphabet to use for the top and bottom strands,\ndefaults to the `Bio.Sequence.Alphabets.Dna.iupac/0`. This allows the most\ngeneral use of the `new` function in unknown scenarios.\n`complement_offset` - Offset for the strands. Positive values are considered\noffset to top, negative as offset to bottom. E.g. `5` would give 5 nt offset\non top, leading to a bottom strand overhand on the 5' side and a top strand\noverhang on the 3' side.","ref":"Bio.Sequence.DnaDoubleStrand.html#new/2-options","title":"Options - Bio.Sequence.DnaDoubleStrand.new/2","type":"function"},{"doc":"A single DNA strand can be represented by the basic sequence which uses\n`Bio.BaseSequence` .\n\n# Examples\n    iex>\"tagc\" in DnaStrand.new(\"ttagct\")\n    true\n\n    iex>alias Bio.Enum, as: Bnum\n    ...>DnaStrand.new(\"ttagct\")\n    ...>|> Bnum.map(&(&1))\n    %DnaStrand{sequence: \"ttagct\", length: 6}\n\n    iex>alias Bio.Enum, as: Bnum\n    ...>DnaStrand.new(\"ttagct\")\n    ...>|> Bnum.slice(2, 2)\n    %DnaStrand{sequence: \"ag\", length: 2}","ref":"Bio.Sequence.DnaStrand.html","title":"Bio.Sequence.DnaStrand","type":"module"},{"doc":"A module for working with RNA.\n\nThis module doesn't contain a representative struct, as with `Bio.Sequence.Dna`.\nThis is because there are multiple ways to interpret a string as RNA. Namely, it\ncan either be single or double stranded. This is why the\n`Bio.Sequence.RnaStrand` and `Bio.Sequence.RnaDoubleStrand` modules exist.\n\nHowever, this is the interface for dealing with things like `complement/1` and\n`reverse_complement/1`.\n\nAdditionally, this module handles defining default conversions for the DNA\nsequence types into RNA sequence types (`Bio.Sequence.DnaStrand` and\n`Bio.Sequence.DnaDoubleStrand`). Conversions defined here are used by the\n`Bio.Sequence.RnaStrand` and `Bio.Sequence.RnaDoubleStrand` modules.\nThe default conversions use conventional nucleotides and map them to their\nrelevant DNA nucleotides:\n\n```\na -> a\nu -> t\ng -> g\nc -> c\n```\n\nCasing is preserved, so mixed case sequences will not be altered.\n\n# Example\n\n    iex>RnaStrand.new(\"uaUUg\")\n    ...>|> Bio.Polymer.convert(DnaStrand)\n    {:ok, %DnaStrand{sequence: \"taTTg\", length: 5}}\n\nThis is guaranteed, so you may encode these with intention and assume that\nthey are preserved across conversions.","ref":"Bio.Sequence.Rna.html","title":"Bio.Sequence.Rna","type":"module"},{"doc":"Provide the RNA complement to a sequence.\n\nGiven a sequence that is either a binary or a `Bio.Sequence.RnaStrand`,\nreturns the RNA complement as defined by the standard nucleotide complements.\n\n# Examples\n    iex>Rna.complement(\"auugacgu\")\n    {:ok, \"uaacugca\"}\n\n    iex>RnaStrand.new(\"auugacgu\")\n    ...>|> Rna.complement()\n    {:ok, %RnaStrand{sequence: \"uaacugca\", length: 8, alphabet: Alpha.common()}}","ref":"Bio.Sequence.Rna.html#complement/2","title":"Bio.Sequence.Rna.complement/2","type":"function"},{"doc":"Provide the RNA reverse complement to a sequence.\n\nGiven a sequence that is either a binary or a `Bio.Sequence.RnaStrand`,\nreturns the RNA reverse complement as defined by the standard nucleotide\ncomplements.\n\n# Examples\n    iex>Rna.reverse_complement(\"auugacgu\")\n    \"acgucaau\"\n\n    iex>RnaStrand.new(\"auugacgu\")\n    ...>|> Rna.reverse_complement()\n    %RnaStrand{sequence: \"acgucaau\", length: 8}","ref":"Bio.Sequence.Rna.html#reverse_complement/1","title":"Bio.Sequence.Rna.reverse_complement/1","type":"function"},{"doc":"","ref":"Bio.Sequence.Rna.html#t:complementable/0","title":"Bio.Sequence.Rna.complementable/0","type":"type"},{"doc":"","ref":"Bio.Sequence.RnaDoubleStrand.html","title":"Bio.Sequence.RnaDoubleStrand","type":"module"},{"doc":"A single RNA strand can be represented by the basic sequence which implements\nthe `Bio.Polymer` behavior.\n\nThis module doesn't implement any validations, since those are not well\ndefined in every case. For example, it may be valid to contain ambiguous\nnucleotides, or it may not. Since that depends on the use, this is left to\napplications developers to write.\n\n# Examples\n    iex>\"uagc\" in  RnaStrand.new(\"uuagcu\")\n    true\n\n    iex>alias Bio.Enum, as: Bnum\n    ...>RnaStrand.new(\"uuagcu\")\n    ...>|> Bnum.map(&(&1))\n    %RnaStrand{sequence: \"uuagcu\", length: 6}\n\n    iex>alias Bio.Enum, as: Bnum\n    ...>RnaStrand.new(\"uuagcu\")\n    ...>|> Bnum.slice(2, 2)\n    %RnaStrand{sequence: \"ag\", length: 2}","ref":"Bio.Sequence.RnaStrand.html","title":"Bio.Sequence.RnaStrand","type":"module"},{"doc":"Amino acids are modeled as simple sequences using `Bio.BaseSequence`.\n\n# Examples\n    iex>aa = AminoAcid.new(\"ymabagta\")\n    ...>\"mabag\" in aa\n    true\n\n    iex>alias Bio.Enum, as: Bnum\n    ...>AminoAcid.new(\"ymabagta\")\n    ...>|>Bnum.map(&(&1))\n    %AminoAcid{sequence: \"ymabagta\", length: 8}\n\n    iex>alias Bio.Enum, as: Bnum\n    ...>AminoAcid.new(\"ymabagta\")\n    ...>|>Bnum.slice(2, 2)\n    %AminoAcid{sequence: \"ab\", length: 2}\n\nIf you are interested in defining conversions of amino acids then look into\nthe `Bio.Polymer` module for how to deal with creating a Conversion module.\n\nThe simple `Bio.Sequence.AminoAcid` does define the `Bio.Polymeric` protocol,\nwhich will allow you to define conversions from this to any type you may\ndesire.","ref":"Bio.Sequence.AminoAcid.html","title":"Bio.Sequence.AminoAcid","type":"module"},{"doc":"Alphabets relevant to the sequences, coding schemes are expressed in\nessentially [BNF](https://en.wikipedia.org/wiki/Backus%E2%80%93Naur_form).\nValues and interpretations for the scheme were accessed from\n[here](https://www.insdc.org/submitting-standards/feature-table/).\n\nAlso exposes the complementary elements for DNA/RNA allowing strands to be\ncomplemented. These functions shouldn't be used directly, but look at\n`Bio.Sequence.Dna.complement/2` and `Bio.Sequence.Rna.complement/1` for more\ninformation.\n\nAlphabets may be used in the declaration of `Bio.BaseSequence` structs to\ndefine how they should be validated. In case one is not supplied, a default\nmay be preferred. See `Bio.Sequence.Dna`, `Bio.Sequence.Rna`,\n`Bio.Sequence.AminoAcid`, and `Bio.Polymer.valid?/2` for more information.\n\n- `Bio.Sequence.Dna`\n  The DNA alphabets provided are:\n  - `common` - The standard bases `ATGCatgc`\n  - `with_n` - The standard alphabet, but with the ambiguous \"any\" character\n  `Nn`\n  - `iupac` - The IUPAC standard values `ACGTRYSWKMBDHVNacgtryswkmbdhvn`\n\n- `Bio.Sequence.Rna`\n  - `common` - The standard bases `ACGUacgu`\n  - `with_n` - The standard alphabet, but with the ambiguous \"any\" character\n  `Nn`\n  - `iupac` - The IUPAC standard values `ACGURYSWKMBDHVNacguryswkmbdhvn`\n\n- `Bio.Sequence.AminoAcid`\n  - `common` - The standad 20 amino acid codes `ARNDCEQGHILKMFPSTWYVarndceqghilkmfpstwyv`\n  - `iupac` - `ABCDEFGHJIKLMNPQRSTVWXYZabcdefghjiklmnpqrstvwxyz`\n\n# Coding Schemes","ref":"Bio.Sequence.Alphabets.html","title":"Bio.Sequence.Alphabets","type":"module"},{"doc":"```\nA ::= Adenine\nC ::= Cytosine\nG ::= Guanine\nT ::= Thymine\n\nR ::= A | G\nY ::= C | T\nS ::= G | C\nW ::= A | T\nK ::= G | T\nM ::= A | C\n\nB ::= S | T (¬A)\nD ::= R | T (¬C)\nH ::= M | T (¬G)\nV ::= M | G (¬T)\nN ::= ANY\n```","ref":"Bio.Sequence.Alphabets.html#module-deoxyribonucleic-acid-codes","title":"Deoxyribonucleic Acid codes - Bio.Sequence.Alphabets","type":"module"},{"doc":"```\nA ::= Adenine\nC ::= Cytosine\nG ::= Guanine\nU ::= Uracil\n\nR ::= A | G\nY ::= C | U\nS ::= G | C\nW ::= A | U\nK ::= G | U\nM ::= A | C\n\nB ::= S | U (¬A)\nD ::= R | U (¬C)\nH ::= M | U (¬G)\nV ::= M | G (¬U)\nN ::= ANY\n```","ref":"Bio.Sequence.Alphabets.html#module-ribonucleic-acid-codes","title":"Ribonucleic Acid codes - Bio.Sequence.Alphabets","type":"module"},{"doc":"```\nA ::= Alanine\nC ::= Cysteine\nD ::= Aspartic Acid\nE ::= Glutamic Acid\nF ::= Phenylalanine\nG ::= Glycine\nH ::= Histidine\nI ::= Isoleucine\nK ::= Lysine\nL ::= Leucine\nM ::= Methionine\nN ::= Asparagine\nP ::= Proline\nQ ::= Glutamine\nR ::= Arginine\nS ::= Serine\nT ::= Threonine\nV ::= Valine\nW ::= Tryptophan\nY ::= Tyrosine\n\nB ::= D | N\nZ ::= Q | E\nJ ::= I | L\nX ::=  ANY\n```","ref":"Bio.Sequence.Alphabets.html#module-amino-acid-codes","title":"Amino Acid codes - Bio.Sequence.Alphabets","type":"module"},{"doc":"Amino Acid Alphabets","ref":"Bio.Sequence.Alphabets.AminoAcid.html","title":"Bio.Sequence.Alphabets.AminoAcid","type":"module"},{"doc":"ARNDCEQGHILKMFPSTWYVarndceqghilkmfpstwyv","ref":"Bio.Sequence.Alphabets.AminoAcid.html#common/0","title":"Bio.Sequence.Alphabets.AminoAcid.common/0","type":"function"},{"doc":"ABCDEFGHJIKLMNPQRSTVWXYZabcdefghJiklmnpqrstvwxyz","ref":"Bio.Sequence.Alphabets.AminoAcid.html#iupac/0","title":"Bio.Sequence.Alphabets.AminoAcid.iupac/0","type":"function"},{"doc":"DNA Alphabets","ref":"Bio.Sequence.Alphabets.Dna.html","title":"Bio.Sequence.Alphabets.Dna","type":"module"},{"doc":"ATGCatgc","ref":"Bio.Sequence.Alphabets.Dna.html#common/0","title":"Bio.Sequence.Alphabets.Dna.common/0","type":"function"},{"doc":"Complements a given character according to the supplied alphabet.\n\nAlphabet must be one of the valid `Bio.Sequence.Alphabets.Dna` options.","ref":"Bio.Sequence.Alphabets.Dna.html#complement/2","title":"Bio.Sequence.Alphabets.Dna.complement/2","type":"function"},{"doc":"ACGTRYSWKMBDHVNacgtryswkmbdhvn","ref":"Bio.Sequence.Alphabets.Dna.html#iupac/0","title":"Bio.Sequence.Alphabets.Dna.iupac/0","type":"function"},{"doc":"ACGTNacgtn","ref":"Bio.Sequence.Alphabets.Dna.html#with_n/0","title":"Bio.Sequence.Alphabets.Dna.with_n/0","type":"function"},{"doc":"RNA Alphabets","ref":"Bio.Sequence.Alphabets.Rna.html","title":"Bio.Sequence.Alphabets.Rna","type":"module"},{"doc":"ACGUacgu","ref":"Bio.Sequence.Alphabets.Rna.html#common/0","title":"Bio.Sequence.Alphabets.Rna.common/0","type":"function"},{"doc":"Complements a given character according to the supplied alphabet.\n\nAlphabet must be one of the valid `Bio.Sequence.Alphabets.Rna` options.","ref":"Bio.Sequence.Alphabets.Rna.html#complement/2","title":"Bio.Sequence.Alphabets.Rna.complement/2","type":"function"},{"doc":"ACGURYSWKMBDHVNZacguryswkmbdhvnz","ref":"Bio.Sequence.Alphabets.Rna.html#iupac/0","title":"Bio.Sequence.Alphabets.Rna.iupac/0","type":"function"},{"doc":"ACGUNacgun","ref":"Bio.Sequence.Alphabets.Rna.html#with_n/0","title":"Bio.Sequence.Alphabets.Rna.with_n/0","type":"function"},{"doc":"Defines behavior for modules to act as a converter between sequences.\n\nThe core of this base module is to provide the default `to/1` function. This\nwill return the error tuple for undefined conversions. This alleviates the\nneed of the user defined module to provide this implementation, and eliminates\nthe possibility of the function `to/1` raising due to no matching clauses.\n\nTo use this as a base for your converter you `use` the module and pass a block\nfor defining the user-side `to/1` calls. For example:\n\n``` elixir\ndefmodule SomeConversion do\n  use Bio.Convertible do\n    def to(SomeModule), do: {:ok, &your_kmer_converter/2, 6}\n\n    defp your_kmer_converter({:ok, kmers, data}, module) do\n      # conversion logic\n    end\n  end\nend\n```\n\nThis defines the k-wise converter that will be used by\n`Bio.Polymer.convert/3`.\n\nThe function you define for the actual conversion will be given an `:ok` tuple\nwith the kmers and any additional data defined for the struct that you're\nconverting. How this data is partitioned is managed by the `Bio.Polymeric`\ndefinition.\n\nAs an example, the `Bio.Sequence.DnaStrand` struct simply drops the `sequence`\nkey, retaining all other keys as the `data` given to the `fn/2` you define.\nThis allows you to retain any relevant information for a newly created struct.\n\nIf you wanted to define your own sequence, this then requires that you also\nimplement the `Bio.Polymeric` interface. So if you were to implement\n`SomeSequence`, you would do the following:\n\n``` elixir\ndefmodule SomeSequence do\n  @behaviour Bio.Sequential\n\n  @impl Bio.Sequential\n  def converter, do: SomeConversion\n\n  # implementation of other callbacks\nend\n```\n\nThe `Bio.Sequential` behavior ensures that we implement the `converter/0`\nfunction which is called from the `Bio.Polymer` module. This in turn\nconstructs the basic converter mechanic, and now you would implement\n`Bio.Polymeric`:\n\n``` elixir\ndefimpl Bio.Polymeric, for: SomeSequence do\n  def kmers(seq, k) do\n    # your logic for splitting the polymer into k sized kmers\n  end\n\n  def valid?(seq, alpha) do\n    # your logic checking if the polymer is valid\n  end\n\n  def validate(seq, alpha) do\n    # your logic for validating the polymer\n  end\nend\n```\n\nNow you can simply call:\n\n``` elixir\nSomeSequence.new(\"some data\")\n|> Bio.Polymer.convert(SomeModule)\n```\n\nThe `Bio.Polymer.convert/3` function now handles calling your conversion\nmethod `your_kmer_converter`. This will be called with the kmers generated by\n`Bio.Polymeric`'s implementation of `kmers/2`, which will be passed a value of\n`k=6`, as defined in the conversion callback.\n\nThis is perhaps overly complex, but here are the design goals:\n\n1. Allow a user to define a conversion from one type to another, regardless of\nif that type is internal.\n2. Allow that definition to leverage existing protocols for e.g. kmers.\n3. Allow a dead simple interface for working with internal components when\nextensions aren't needed.\n\nA lot of complexity was pulled in to hit these targets, and I think that the\ngoal is achieved. A user who doesn't need their own types can simply work with\nthe `Bio.Sequence` types using the `Bio.Polymer` interface.\n\nHowever, if they want to define `e.g.` a converter between\n`Bio.Sequence.RnaStrand` and `Bio.Sequence.AminoAcid`, they can simply define\na module for that conversion and pass it to the `Bio.Polymer.convert/3`\nfunction.\n\nThe complexity only arises to the user when they need their own sequences.\nHopefully they'd have appreciated the necessity of the complexity before then.","ref":"Bio.Convertible.html","title":"Bio.Convertible","type":"behaviour"},{"doc":"Defines the converter's k-wise conversion function\n\nThis is called within the `Bio.Polymer.convert/3` function to acquire the\nk-wise conversion function for sequence to another.","ref":"Bio.Convertible.html#c:to/1","title":"Bio.Convertible.to/1","type":"callback"},{"doc":"How a \"sequence\" ought to comport itself.\n\nThe `Bio.Sequential` behaviour is used to define the expected\nfunctions exposed by \"sequences\". In general, sequences are basically a\nreplacement for the enumerable qualities of strings in some other languages.\n\nBecause that requires defining a protocol, and that requires a struct, it\nmakes a lot of sense to have an easy to use initializer. Thus, the `new/2`\nmethod. The `opts` defined will depend on the type of sequence you're\ncreating, and so the typing is left rather general here. These should be\nrefined for any concrete implementation.\n\nBecause most sequences can be transcoded into other sequences (e.g. DNA ->\nAmino Acid), we also want to define a getter for the module that handles\nthat conversion. This is why the `converter/0` function exists. It should\nreturn a module that implements the `Bio.Convertible` behaviour.\n\nTogether with the `Bio.Polymeric` protocol, the `Bio.Convertible` behaviour\nand `Bio.Polymer` module  create a robust conversion mechanic that can be\nhooked into by user defined types. For further reading on that, look at the\n`Bio.Polymer` module docs.\n\nThe final callback, `fasta_line/1`, exists because this is a bioinformatics\nlibrary. Sequences are pretty much always going to be written out to a fasta\nfile, or some similar context. Defining this as a callback means that we can\nmake it easier for your types to be given directly to the `Bio.IO.Fasta`\nmodule for writing. Eventually, I'd probably like to come up with a more\ngeneral `dump` style mechanic. But this'll do for the pre-alpha.","ref":"Bio.Sequential.html","title":"Bio.Sequential","type":"behaviour"},{"doc":"Returns the module which implements `to/1` functions for type conversion.\n\nThis returns the module, which is then used from within\n`Bio.Polymer.convert/3` to acquire the correct conversion function\nfor a given type.\n\nYour code shouldn't need to call this. It exists to interface with the\n`Bio.Polymer` module's functions.","ref":"Bio.Sequential.html#c:converter/0","title":"Bio.Sequential.converter/0","type":"callback"},{"doc":"Given a struct, returns the String.t() line for a FASTA file\n\nThis will be called from within `Bio.IO.Fasta.write/3`","ref":"Bio.Sequential.html#c:fasta_line/1","title":"Bio.Sequential.fasta_line/1","type":"callback"},{"doc":"Builds a new struct for the implementing type.\n\nThis is provided as:\n1. A simple way to initialize various types.\n2. A place to perform pre-computations/optimizations in the future.\n\nThe default implementation for example does a pre-computation of the length of\nthe sequence, which is stored on the base type. This in turn allows the\n`Enumerable` interface to be implemented efficiently.","ref":"Bio.Sequential.html#c:new/2","title":"Bio.Sequential.new/2","type":"callback"},{"doc":"Implements a wrapper around the `Enum` module's public interface.\n\nThe semantics of the `Enum` module don't always match up with what I would think\nis best for certain cases. The best example of this is the `slide/3` function.\nBecause of the `Enum` implementation, there is no way to coerce the return\nvalue back into a struct. So for example, given a `Bio.Sequence.DnaStrand` it\nwould return a list of graphemes. This is not what I want users to expect.\n\nThat said, there are other functions that _do_ behave well. Or at the very\nleast, their semantics seem meaningfully useful. So in order to preserve the\nmaximum utility, I wrap the module.\n\nThe expectation should be as follows:\n`Enum` functions will return bare data.\n`Bio.Enum` functions will return the closest thing to the struct as is\nreasonable.\n\nThere are cases where it doesn't make much sense to return more than is\nrequired. For example, the `Bio.Enum.at/2` function will return a binary\ngrapheme. I have a hard time imagining a case where the user would want a\nstruct with a sequence of a single character instead of the character itself.\n\nContrast that with the `Enum.at/2` function, which will return a raw char.","ref":"Bio.Enum.html","title":"Bio.Enum","type":"module"},{"doc":"","ref":"Bio.Enum.html#at/2","title":"Bio.Enum.at/2","type":"function"},{"doc":"","ref":"Bio.Enum.html#at/3","title":"Bio.Enum.at/3","type":"function"},{"doc":"","ref":"Bio.Enum.html#chunk_every/2","title":"Bio.Enum.chunk_every/2","type":"function"},{"doc":"","ref":"Bio.Enum.html#chunk_every/3","title":"Bio.Enum.chunk_every/3","type":"function"},{"doc":"","ref":"Bio.Enum.html#chunk_every/4","title":"Bio.Enum.chunk_every/4","type":"function"},{"doc":"","ref":"Bio.Enum.html#chunk_while/4","title":"Bio.Enum.chunk_while/4","type":"function"},{"doc":"","ref":"Bio.Enum.html#map/2","title":"Bio.Enum.map/2","type":"function"},{"doc":"","ref":"Bio.Enum.html#reverse/1","title":"Bio.Enum.reverse/1","type":"function"},{"doc":"","ref":"Bio.Enum.html#reverse/2","title":"Bio.Enum.reverse/2","type":"function"},{"doc":"","ref":"Bio.Enum.html#slice/2","title":"Bio.Enum.slice/2","type":"function"},{"doc":"","ref":"Bio.Enum.html#slice/3","title":"Bio.Enum.slice/3","type":"function"},{"doc":"","ref":"Bio.Enum.html#t:acc/0","title":"Bio.Enum.acc/0","type":"type"},{"doc":"","ref":"Bio.Enum.html#t:default/0","title":"Bio.Enum.default/0","type":"type"},{"doc":"","ref":"Bio.Enum.html#t:element/0","title":"Bio.Enum.element/0","type":"type"},{"doc":"","ref":"Bio.Enum.html#t:index/0","title":"Bio.Enum.index/0","type":"type"},{"doc":"","ref":"Bio.Enum.html#t:t/0","title":"Bio.Enum.t/0","type":"type"},{"doc":"Define Polymeric interface of a sequence type.\n\nThe `Bio.Polymeric` protocol allows us to define implementations\nof a `kmers/2` function. This is part of the approach to translating different\npolymers according to the nature of actual biological or chemical processes.\n\nThe idea is that defining how a sequence is sub-divided into k-mers for\nenumeration is something that must occur for specific conversions. However,\nit's also something that you would not necessarily want to have to do every\nsingle time you applied the conversion.\n\nEssentially, each structural definition of a sequence will have some\nmeaningful way of splitting it into a Kmer enumeration. This is used in all\nforms of computation, largely though, in conversions. For example, DNA -> RNA\nconversions require element-wise (k=1) conversion functions. Whereas, RNA ->\nAmino Acid requires codon-wise (k=3).\n\nIn order to preserve the standard interface defined by `Bio.Polymer`\nand `Bio.Polymer.convert/3`, we define this as a protocol.\n\nFor a valid return, the consideration should be:\n1. The enumerable returned (`Enum.t()`) should contain the information\nrequired to perform a conversion. Examples can be found in the\n`Bio.Sequence.DnaStrand` and `Bio.Sequence.DnaDoubleStrand` modules. There,\nyou'll see that for a simple sequence, it makes sense to simple iterate the\ngrouped chunks. Whereas the double stranded sequence returns a list of tuples\nof chunks.\n2. The `map()` should contain relevant data for the re-capitulation of a\nstruct. So if you're converting a `DnaStrand`, you should consider passing\nback out the `label` field. This allows the conversion function to attach it\nto the newly constructed type.\n\nThe error mode for various sequences will vary, but generally the idea of\nmismatching the sequence length to the `k` value will hold. For the build in\n`Bio.Sequence.DnaStrand`, this is merely the even division. For the\n`Bio.Sequence.DnaDoubleStrand` it's more complicated. That type assumes that\nyou want to see pairs of aggregated values (top/bottom), but they may be\noffset. So you can't just look at if the values are empty.\n\nInstead, it looks to see if there can be complete aggregates, even if they're\npaired with empty space.\n\nKeep these considerations in mind implementing your own `Polymeric` types.\n\nIn addition to the enumeration of the elements, this also makes sense as the\nlocation for defining validity. That is, there are two further methods\n`valid?/2` and `validate/2`.\n\nThese make the assumption that a relevant `alphabet` is defined for the\npolymer. For example, [IUPAC DNA\nCodes](https://en.wikipedia.org/wiki/Nucleic_acid_notation).\n\nYour implementation of `valid?/2` and `validate/2` should prefer the alphabet\ngiven to them. This will be respected by the `Bio.Polymer.valid?/2` and\n`Bio.Polymer.validate/2` function. Essentially, when used, these will always\nprefer the given value, but will default back to the value attached to the\ntype if it is defined.\n\n# Example\n    iex>alias Bio.Sequence.Alphabets.Dna, as: Alpha\n    ...>Bio.Sequence.DnaStrand.new(\"atgcnn\", alphabet: Alpha.common())\n    ...>|> Bio.Polymer.valid?()\n    false\n\n    iex>alias Bio.Sequence.Alphabets.Dna, as: Alpha\n    ...>Bio.Sequence.DnaStrand.new(\"atgcnn\", alphabet: Alpha.common())\n    ...>|> Bio.Polymer.valid?(Alpha.with_n())\n    true\n\n> ##","ref":"Bio.Polymeric.html","title":"Bio.Polymeric","type":"protocol"},{"doc":"> In case neither is defined, the `validate/2` function will return an error\n> tuple, where the `valid?` will simply return false.\n\nThe `validate/2` function behaves similarly, but it should return a new struct\nwith the `valid?` key set.\n\n# Example\n    iex>alias Bio.Sequence.Alphabets.Dna, as: Alpha\n    ...>Bio.Sequence.DnaStrand.new(\"atgcnn\", alphabet: Alpha.common())\n    ...>|> Bio.Polymer.validate()\n    {\n      :error,\n      [{:mismatch_alpha, \"n\", 4}, {:mismatch_alpha, \"n\", 5}]\n    }\n\n    iex>alias Bio.Sequence.Alphabets.Dna, as: Alpha\n    ...>Bio.Sequence.DnaStrand.new(\"atgcnn\", alphabet: Alpha.common())\n    ...>|> Bio.Polymer.validate(Alpha.with_n())\n    {\n      :ok,\n      %Bio.Sequence.DnaStrand{\n        sequence: \"atgcnn\",\n        length: 6,\n        alphabet: \"ACGTNacgtn\",\n        valid?: true\n      }\n    }\n\n\n> ##","ref":"Bio.Polymeric.html#module-note-neutral","title":"Note {: .neutral} - Bio.Polymeric","type":"protocol"},{"doc":"> The applied alphabet is the one that is returned in the struct. This ensures\n> that you are correctly tracking what a type is valid _for_. So be careful\n> about assumptions.","ref":"Bio.Polymeric.html#module-note-neutral","title":"Note {: .neutral} - Bio.Polymeric","type":"protocol"},{"doc":"Split a polymer into chunks of `k` size","ref":"Bio.Polymeric.html#kmers/2","title":"Bio.Polymeric.kmers/2","type":"function"},{"doc":"Determine if the content of a polymer matches an alphabet","ref":"Bio.Polymeric.html#valid?/2","title":"Bio.Polymeric.valid?/2","type":"function"},{"doc":"Validate if the content of a polymer matches an alphabet, returning an updated\nstruct.\n\nDepends on the struct implementing both an `alphabet` and `valid?` keys.","ref":"Bio.Polymeric.html#validate/2","title":"Bio.Polymeric.validate/2","type":"function"},{"doc":"","ref":"Bio.Polymeric.html#t:t/0","title":"Bio.Polymeric.t/0","type":"type"},{"doc":"Internal helper module for dealing with amino acids","ref":"Bio.AcidHelper.html","title":"Bio.AcidHelper","type":"module"},{"doc":"","ref":"Bio.AcidHelper.html#t:alphabet/0","title":"Bio.AcidHelper.alphabet/0","type":"type"},{"doc":"","ref":"Bio.AcidHelper.html#t:character/0","title":"Bio.AcidHelper.character/0","type":"type"},{"doc":"","ref":"Bio.AcidHelper.html#t:index/0","title":"Bio.AcidHelper.index/0","type":"type"},{"doc":"","ref":"Bio.AcidHelper.html#t:mismatch/0","title":"Bio.AcidHelper.mismatch/0","type":"type"}]}